C$Procedure      FITCK3 ( Fit a orientations with a type 3 C-kernel )
 
      SUBROUTINE FITCK3 ( CAVSRC, REPORT, TOL,   SCID,
     .                    HANDLE, BEGET,  ENDET, PLTFRM,  REF, SEGID )
 
C$ Abstract
C
C     Given a routine that will provide quaternions and angular
C     velocity of some structure with respect to some reference
C     frame, construct one or more type 3 C-kernel segmenst that
C     agrees with the source data to a user specified tolerance.
C
C$ Copyright
C
C     Copyright (1997), California Institute of Technology.
C     U.S. Government sponsorship acknowledged.
C
C$ Required_Reading
C
C      None.
C
C$ Keywords
C
C      CK
C
C$ Declarations
 
      IMPLICIT NONE
      EXTERNAL              CAVSRC
      EXTERNAL              REPORT
      DOUBLE PRECISION      TOL
      INTEGER               SCID
      INTEGER               HANDLE
      DOUBLE PRECISION      BEGET
      DOUBLE PRECISION      ENDET
      INTEGER               PLTFRM
      CHARACTER*(*)         REF
      CHARACTER*(*)         SEGID
 
C$ Brief_I/O
C
C     VARIABLE  I/O  DESCRIPTION
C     --------  ---  --------------------------------------------------
C     CAVSRC     I   is a routine that given an ET will produce Q and AV
C     REPORT     I   is a routine that reports progress of the work.
C     TOL        I   is the accuracy required of the output C-kernel
C     SCID       I   is the SCLK id-code to use.
C     HANDLE     I   is the handle of a C-kernel open for writing
C     BEGET      I   is the beginning ET for the C-kernel
C     ENDET      I   is the ending ET for the C-kernel
C     PLTFRM     I   is the id-code for the platform being modelled
C     REF        I   is the reference frame to associate with the CK
C     SEGID      I   is the string to attach to each segment created
C
C$ Detailed_Input
C
C     CAVSRC     is a routine that given an ET will produce Q and AV.
C                The calling sequence should be:
C
C                   SUBROUTINE CAVSRC ( ET, CMATRX, AV )
C
C                the only input is ET.  The outputs are the CMATRX
C                and AV (C-matrix and Angular Velocity)
C                to associate with the structure specified by PLTFRM
C                relative to the reference frame specified by REF.
C
C                It is probably a good design practice to create an
C                umbrella routine that contains an initialization
C                entry point and the entry point that performs the
C                function of CAVSRC.  In this way, you can set up any
C                particular parameters needed by CAVSRC prior to calling
C                this routine.  When CAVSRC is called by FITCK3, it
C                can simply look up these parameters from the variables
C                shared by the initialization entry point and the
C                entry point corresponding to CAVSRC.
C
C     REPORT     is a routine that given the last ET processed by FITCK3
C                will produce a suitable progress report.  The calling
C                sequence to REPORT should be
C
C                    SUBROUTINE REPORT ( ET )
C
C                As with CAVSRC, this assumes that REPORT is an entry
C                point in a routine that contains another initialization
C                entry point that sets up the parameters needed for
C                producing a meaningful report.
C
C                If you don't care about progress reports in your
C                application, just make a stub routine that simply
C                returns without taking any action.
C
C     TOL        is the accuracy required of the output C-kernel.  The
C                output C-kernel will match CAVSRC at every ET to the
C                accuracy specified by TOL.  TOL is an angle in
C                radians.  The orientation of the C-kernel will match
C                the outputs of CAVSRC to within and angle of TOL.
C                The angular velocity vectors will also be within
C                TOL radians of being parallel.
C
C     SCID       is the SCLK id to use when converting ET to ticks.
C                Normally this should be the id-code of the spacecraft
C                that the structure is mounted to.  However, in some
C                cases, another clock id may be approrpriate.
C
C     HANDLE     is the handle of a C-kernel that has been opened
C                for writing in which FITCK3 will insert new C-kernel
C                segments.
C
C     BEGET      is the initial epoch for the C-kernel segment(s) that
C                will be produced by this routine.
C
C     ENDET      is the last epoch for the C-kernel segment(s) that
C                will be produced by this routine.
C
C     PLTFRM     is the ID-code of the object whose orientation is to
C                be captured in the C-kernel.
C
C     REF        is the reference frame to associate with the C-kernel
C                segments created by this routine.
C
C     SEGID      is the segment identifier to be used for segments
C                created by this routine.  Note that the same segment
C                identifier will be used for all segments that are
C                created by this routine.
C
C$ Detailed_Output
C
C     None.
C
C$ Parameters
C
C     None.
C
C$ Files
C
C     None.
C
C$ Exceptions
C
C     1)  If the routine CAVSRC modifies the input value of ET that
C         it receives without changing resetting it before returning,
C         the error  'SPICE(BADPASSEDROUTINE)' is signalled.
C
C     2)  Any problems that arise the construction of the new
C         C-kernel data are diagnosed by routines called by this
C         routine.
C
C$ Particulars
C
C     This routine is intended to handle the normal logic needed
C     when creating a C-kernel that fits some continuous source
C     of orientation data.
C
C$ Examples
C
C     Later.
C
C$ Restrictions
C
C     None.
C
C$ Author_and_Institution
C
C     W.L. Taber      (JPL)
C
C$ Literature_References
C
C     None.
C
C$ Version
C
C-    SPICELIB Version 3.0.0, 25-FEB-1998 (WLT)
C
C        Added code to short circuit SPICELIB error handling
C        so that extra information can be added to the long
C        message if one is signalled in the call to CAVSRC.
C
C        In addition if the value of ET supplied to CAVSRC
C        comes back changed, and error is signalled indicating
C        that CAVSRC is not performing to specification.
C
C-    SPICELIB Version 2.1.0, 25-FEB-1998 (WLT)
C
C        Added aliasing checks and checks to make sure the
C        initial STEP away from BEGET is not too large.
C
C-    SPICELIB Version 2.0.0, 30-JAN-1998 (WLT)
C
C        Reworked the basic logic of the routine with the intent
C        of making the meaning of various branches clearer
C
C-    SPICELIB Version 1.0.0, 24-JUL-1997 (WLT)
C
C
C-&
 
C$ Index_Entries
C
C     Fit type 3 C-kernel to a continuous source of orientation
C
C-&
C
C     SPICELIB Functions
C
      DOUBLE PRECISION      VNORM
      DOUBLE PRECISION      PI
      DOUBLE PRECISION      DPR
 
      LOGICAL               RETURN
      LOGICAL               FAILED
 
C
C     Parameters
C
C     QBUFSZ is the number of quaternians we buffer before writing
C     a segment.
C
      INTEGER               QBUFSZ
      PARAMETER           ( QBUFSZ = 10000 )
C
C     NCHECK is the number of interpolation checks we perform per step.
C
 
      INTEGER               NCHECK
      PARAMETER           ( NCHECK = 6 )
 
      DOUBLE PRECISION      MNPTS
      PARAMETER           ( MNPTS = 20.0D0 )
 
      DOUBLE PRECISION      MININC
      PARAMETER           ( MININC = 20.0D0 )
 
      DOUBLE PRECISION      MINSTP
      PARAMETER           ( MINSTP = 1.0D0 )
 
      INTEGER               WDSIZE
      PARAMETER           ( WDSIZE = 32 )
 
      INTEGER               LNSIZE
      PARAMETER           ( LNSIZE = 80 )
 
 
C
C     Local Variables
C
      CHARACTER*(WDSIZE)    ACTION
      CHARACTER*(WDSIZE)    DEVICE
      CHARACTER*(WDSIZE)    BSTR
      CHARACTER*(WDSIZE)    SSTR
      CHARACTER*(WDSIZE)    ESTR
      CHARACTER*(WDSIZE)    SHORT
      CHARACTER*(WDSIZE)    STEPC
      CHARACTER*(WDSIZE)    TOLC
 
      CHARACTER*(LNSIZE)    LINE
 
 
      INTEGER               LONGSZ
      PARAMETER           ( LONGSZ = 3000 )
 
      CHARACTER*(LONGSZ)    MESSG
 
      DOUBLE PRECISION      ANGLE
      DOUBLE PRECISION      AV     ( 3 )
      DOUBLE PRECISION      AVAPRX ( 3 )
      DOUBLE PRECISION      AVFRAC ( 3 )
      DOUBLE PRECISION      AVEL   ( 3, QBUFSZ )
      DOUBLE PRECISION      AVLAST ( 3 )
      DOUBLE PRECISION      AVSTEP ( 3 )
      DOUBLE PRECISION      AVTMP  ( 3 )
      DOUBLE PRECISION      AXIS   ( 3 )
      DOUBLE PRECISION      BEGTIK
      DOUBLE PRECISION      COPY
      DOUBLE PRECISION      CMAPRX ( 3, 3 )
      DOUBLE PRECISION      CMATRX ( 3, 3 )
      DOUBLE PRECISION      CMFRAC ( 3, 3 )
      DOUBLE PRECISION      CMLAST ( 3, 3 )
      DOUBLE PRECISION      CMSTEP ( 3, 3 )
      DOUBLE PRECISION      DIFF
      DOUBLE PRECISION      ENDTIK
      DOUBLE PRECISION      ENUF
      DOUBLE PRECISION      ETFRST
      DOUBLE PRECISION      ETFRAC
      DOUBLE PRECISION      ETLAST
      DOUBLE PRECISION      ETSTEP
      DOUBLE PRECISION      FRAC
      DOUBLE PRECISION      INCR
      DOUBLE PRECISION      Q      ( 4, QBUFSZ )
      DOUBLE PRECISION      RDIFF  ( 3, 3 )
      DOUBLE PRECISION      REMAIN
      DOUBLE PRECISION      STARTS(1)
      DOUBLE PRECISION      STEP
      DOUBLE PRECISION      TICKS ( QBUFSZ )
 
      DOUBLE PRECISION      STPSCL
      DOUBLE PRECISION      INCSCL
      DOUBLE PRECISION      RATSCL
 
 
      INTEGER               QCOUNT
      INTEGER               J
      INTEGER               MIDPT
      INTEGER               NINTS
      INTEGER               OFFSET
      INTEGER               I
      INTEGER               TQ
      INTEGER               INC
      INTEGER               DIR
 
 
 
      LOGICAL               AVFLAG
      LOGICAL               GOTONE
      LOGICAL               OK
      LOGICAL               TOOBIG
 
 
 
C
C     Standard SPICE error handling.
C
      IF ( RETURN() ) THEN
         RETURN
      END IF
 
      CALL CHKIN ( 'FITCK3' )
 
C
C     Set up the error handling to give this routine local
C     control of message before aborting.
C
      CALL ERRACT ( 'GET',  ACTION  )
      CALL ERRDEV ( 'GET',  DEVICE  )
 
      CALL DPFMT  ( TOL*DPR(), 'xxxx.xxxx', TOLC )
      CALL TOSTDO ( 'Tolerance: ' // TOLC // ' Degrees' )
 
      CALL DPFMT ( TOL, 'xxxx.xxxx', TOLC )
      CALL TOSTDO ( 'Tolerance: ' // TOLC // ' Radianss' )
 
 
 
      ETFRST = BEGET
      ENUF   = (ENDET - BEGET)/MNPTS
      ENUF   = 3600.0D0
 
C
C     Fetch the first pointing instance.
C
      COPY = ETFRST
 
      CALL CHKIN  ( 'CAVSRC-0' )
 
         CALL ERRACT ( 'SET', 'RETURN' )
         CALL ERRDEV ( 'SET', 'NULL'   )
         CALL CAVSRC ( COPY, CMATRX, AV )
 
         IF ( FAILED() ) THEN
 
            CALL GETLMS ( MESSG )
            CALL GETSMS ( SHORT  )
 
            CALL RESET
 
            CALL ERRACT ( 'SET', ACTION )
            CALL ERRDEV ( 'SET', DEVICE )
 
            CALL SUFFIX ( 'The input value of ET to CAVSRC was # '
     .      //            '(#).', 8, MESSG )
 
            CALL ETCAL ( ETFRST, SSTR )
 
            CALL RESET
            CALL SETMSG ( MESSG )
            CALL ERRDP  ( '#', ETFRST )
            CALL ERRCH  ( '#', SSTR   )
            CALL SIGERR ( SHORT       )
            CALL CHKOUT ( 'CAVSRC-0'  )
            CALL CHKOUT ( 'FITCK3'    )
            RETURN
 
         END IF
 
      CALL ERRACT ( 'SET', ACTION )
      CALL ERRDEV ( 'SET', DEVICE )
      CALL CHKOUT ( 'CAVSRC-0' )
 
      IF ( ETFRST .NE. COPY ) THEN
 
         CALL ERRACT ( 'SET', ACTION )
         CALL ERRDEV ( 'SET', DEVICE )
 
         DIFF = ETFRST - COPY
 
         CALL SETMSG ( 'The external routine "CAVSRC" passed '
     .   //            'to FITCK3 modified the input value of '
     .   //            'ETFRST.  The original value of ETFRST '
     .   //            'was #. It was modified to #. The '
     .   //            'difference in these values is #. ' )
         CALL ERRDP  ( '#', ETFRST )
         CALL ERRDP  ( '#', COPY   )
         CALL ERRDP  ( '#', DIFF   )
         CALL SIGERR ( 'SPICE(BADPASSEDROUTINE)'  )
         CALL CHKOUT ( 'FITCK3' )
         RETURN
 
      END IF
 
 
 
 
      QCOUNT = 1
      TQ     = 1
 
C
C     We don't have a bracketing C-matrix yet.
C
      GOTONE = .FALSE.
C
C     Copy the first quaternion and angular velocity into
C     the holding area.
C
      CALL M2Q    ( CMATRX,       Q    (1,QCOUNT) )
      CALL VEQU   ( AV,           AVEL (1,QCOUNT) )
      CALL SCE2T  ( SCID, ETFRST, TICKS(  QCOUNT) )
 
C
C     Here's how we are going to handle this.
C
C
C           |<----STEP---->|
C        ---*--------------*-----------
C           ETFRST         ETSTEP
C
C     We will compute the rotation and angular velocity at ETFRST (we've
C     already done that) and STEP seconds later.  Given this pair
C     of rotations, we'll interpolate several orientations between
C     ETFRST and ETSTEP.  If the interpolated orientations match to
C     within the specified tolerance the actual orientations at
C     these points, we'll
C
C           |<----------STEP-------->|
C        ---*--------------*---------*-
C           ETFRST         ETLAST    ETSTEP
C
C       1) store our last successful interval endpoint in ETLAST;
C       2) increase STEP
C       3) Compute the actual orientation at the new ETSTEP
C       4) perform the checks all overagain.
C
C     This loop continues until we don't match the specified
C     tolerance. At this point we:
C
C
C       1) buffer the time and orientation information at ETLAST.
C       2) advance the reference ETFRST to be where we had ETLAST
C       3) decrease STEP by a small amount
C       4) start the above process all over agin.
C
C
C        Transition from this ...
C
C           |<----------STEP-------->|
C        ---*--------------*---------*---------------
C           ETFRST         ETLAST    ETSTEP
C
C
C        to this.   Note ETFRST moves and STEP shrinks.
C
C                          |<----STEP---->|
C        ---*--------------*--------------*---------
C                          ETFRST         ETSTEP
C
C
      STPSCL = 0.75D0
      INCSCL = 0.05D0
      RATSCL = STPSCL / INCSCL
 
 
      IF ( VNORM(AV) .GT. 0 ) THEN
         STEP = MAX( MININC, STPSCL*TOL/VNORM(AV) )
         INCR = MAX( MININC, INCSCL*TOL/VNORM(AV) )
      ELSE
         STEP = MAX( MININC, ENUF        )
         INCR = MAX( MININC, ENUF/RATSCL )
      END IF
 
      CALL DPFMT ( STEP, 'XXXXXXXX.XXX', STEPC )
      LINE = 'Initial stepsize: # seconds.'
      CALL REPMC ( LINE, '#', STEPC, LINE )
      CALL TOSTDO ( LINE )
 
      ETSTEP = MIN ( ETFRST + STEP, ENDET )
      ETLAST = ETFRST
      STEP   = MIN ( STEP, ENDET - ETLAST )
 
C
C     Since we'll always have a good C-matrix at ETLAST, we
C     compare ETLAST to ENDET so that one loop can handle all
C     of the tests for goodness of fit.  (As will be evident below,
C     the loop will terminate when ETLAST is equal to ENDET.)
C
      DO WHILE ( ETLAST .LT. ENDET )
 
C
C        Get the C-matrix at the next step point.
C
         IF ( ETSTEP .GT. ENDET .OR. ETSTEP .LT. BEGET ) THEN
 
            CALL SETMSG ( 'The current value of ETSTEP is '
     .      //            'outside the specified BEGET to ENDET '
     .      //            'range supplied to FITCK3.  Here are '
     .      //            'the epochs. BEGET = ''#'' ETSTEP = '
     .      //            '''#'' ENDET = ''#'' ' )
 
            CALL ETCAL ( BEGET,  BSTR )
            CALL ETCAL ( ETSTEP, SSTR )
            CALL ETCAL ( ENDET,  ESTR )
 
            CALL ERRCH  ( '#', BSTR )
            CALL ERRCH  ( '#', SSTR )
            CALL ERRCH  ( '#', SSTR )
 
            CALL SIGERR ( 'SPICE(SPICEBUG)'  )
            CALL CHKOUT ( 'FITCK3'           )
            RETURN
 
         END IF
 
         COPY = ETSTEP
 
         CALL CHKIN  ( 'CAVSRC-1' )
 
            CALL ERRACT ( 'SET', 'RETURN' )
            CALL ERRDEV ( 'SET', 'NULL'   )
            CALL CAVSRC ( COPY, CMSTEP, AVSTEP )
 
            IF ( FAILED() ) THEN
               CALL GETLMS ( MESSG )
               CALL GETSMS ( SHORT  )
 
               CALL RESET
 
               CALL ERRACT ( 'SET', ACTION )
               CALL ERRDEV ( 'SET', DEVICE )
 
               CALL SUFFIX ( 'The input value of ET to CAVSRC '
     .         //            'was # (#). ', 8, MESSG )
 
 
               CALL ETCAL ( ETSTEP, SSTR )
 
               CALL RESET
               CALL SETMSG ( MESSG )
               CALL ERRDP  ( '#', ETSTEP )
               CALL ERRCH  ( '#', SSTR   )
               CALL SIGERR ( SHORT       )
               CALL CHKOUT ( 'CAVSRC-1'  )
               CALL CHKOUT ( 'FITCK3'    )
               RETURN
            END IF
 
         CALL ERRACT ( 'SET', ACTION )
         CALL ERRDEV ( 'SET', DEVICE )
         CALL CHKOUT ( 'CAVSRC-1' )
 
         IF ( ETSTEP .NE. COPY ) THEN
 
            CALL ERRACT ( 'SET', ACTION )
            CALL ERRDEV ( 'SET', DEVICE )
 
            DIFF = ETSTEP - COPY
 
            CALL SETMSG ( 'The external routine "CAVSRC" passed '
     .      //            'to FITCK3 modified the input value of '
     .      //            'ETSTEP.  The original value of ETSTEP '
     .      //            'was #. It was modified to #. The '
     .      //            'difference in these values is #. ' )
            CALL ERRDP  ( '#', ETSTEP )
            CALL ERRDP  ( '#', COPY   )
            CALL ERRDP  ( '#', DIFF   )
            CALL SIGERR ( 'SPICE(BADPASSEDROUTINE)'  )
            CALL CHKOUT ( 'FITCK3' )
            RETURN
 
         END IF
 
 
C
C        Compare the interpolated C-matrix with the actual
C        nadir C-matrix at several points in the interval
C        between ETFRST and ETSTEP.  We check first
C        at the midpoint based on the assumption that we
C        are likely to have the worst error at the midpoing.
C
         OK     = .TRUE.
         I      =  1
         OFFSET =  0
         MIDPT  = (NCHECK + 1) / 2
         DIR    =  1
         INC    =  1
 
         DO WHILE ( I .LT.  NCHECK .AND. OK  .AND. STEP .GT. MINSTP )
C
C           Get the fractional position between our two matrices
C           at which we are going to perform the interpolation.
C
            J      = MIDPT + DIR*OFFSET
            FRAC   = DBLE(J)/DBLE(NCHECK + 1)
            REMAIN = 1.0D0  - FRAC
            ETFRAC = ETFRST + FRAC*STEP
 
            IF ( ETLAST .LT. ETFRAC .AND. ETFRAC .LT. ETSTEP ) THEN
C
C           Get the interpolated C-matrix and the actual C-matrix.
C
               CALL MXMT     ( CMSTEP,CMATRX, RDIFF )
               CALL RAXISA   ( RDIFF, AXIS,   ANGLE )
 
               TOOBIG = ANGLE .GT. 0.75D0*PI()
 
               CALL LINROT_M ( CMATRX, CMSTEP, FRAC,   CMAPRX, AVTMP  )
               CALL VLCOM    ( REMAIN, AV,     FRAC,   AVSTEP, AVAPRX )
 
               COPY = ETFRAC
 
               CALL CHKIN  ( 'CAVSRC-2' )
 
               CALL ERRACT ( 'SET', 'RETURN' )
               CALL ERRDEV ( 'SET', 'NULL'   )
 
               CALL CAVSRC ( COPY, CMFRAC, AVFRAC )
               IF ( FAILED() ) THEN
                  CALL GETLMS ( MESSG )
                  CALL GETSMS ( SHORT  )
 
                  CALL RESET
 
                  CALL ERRACT ( 'SET', ACTION )
                  CALL ERRDEV ( 'SET', DEVICE )
 
                  CALL SUFFIX ( 'The input value of ET to CAVSRC '
     .            //            'was # (#). ', 8, MESSG )
 
 
                  CALL ETCAL ( ETFRAC, SSTR )
 
                  CALL RESET
                  CALL SETMSG ( MESSG )
                  CALL ERRDP  ( '#', ETFRAC )
                  CALL ERRCH  ( '#', SSTR   )
                  CALL SIGERR ( SHORT       )
                  CALL CHKOUT ( 'CAVSRC-2'  )
                  CALL CHKOUT ( 'FITCK3'    )
                  RETURN
 
               END IF
 
               CALL ERRACT ( 'SET', ACTION )
               CALL ERRDEV ( 'SET', DEVICE )
               CALL CHKOUT ( 'CAVSRC-2' )
 
 
 
 
C
C              Find the size of the rotation from the interpolated
C              matrix to the real matrix.
C
               CALL MXMT     ( CMFRAC, CMAPRX, RDIFF )
               CALL RAXISA   ( RDIFF,  AXIS,   ANGLE )
 
 
C
C              As long as ANGLE remains below TOL we will not need
C              to store anything.
C
               OK =        TOL .GE. ANGLE
     .               .AND. TOL .GE. DABS (VNORM(AVAPRX) - VNORM(AVFRAC))
     .               .AND. .NOT. TOOBIG
 
            END IF
 
            INC    = INC + 1
            DIR    = -DIR
            OFFSET = INC/2
            I      = I + 1
 
         END DO
C
C        How did the above loop terminate?
C
         IF ( OK ) THEN
C
C           All the interpolations match to the specified tolerance.
C           Make the stepsize a bit bigger and store the value of the
C           C-matrix and angular velocity that were computed at ETSTEP.
C
C           Note:  In the DO WHILE ( ETLAST .LT. ENDET ) loop that
C           encloses the IF block we're in now, this
C           is the only location where ETLAST has its value modified.
C
            GOTONE = .TRUE.
            ETLAST = ETSTEP
            STEP   = STEP  + INCR
            CALL     VEQUG ( CMSTEP, 9, CMLAST )
            CALL     VEQU  ( AVSTEP,    AVLAST )
 
         ELSE IF ( .NOT. GOTONE ) THEN
C
C           There's only one way to get here.  The initial step
C           away from ETFRST was too big.  We need to shrink things
C           down a bit.
C
            IF ( STEP .LE. MININC ) THEN
               STEP = MAX( MINSTP, STEP / DSQRT(2.0D0) )
               INCR = STEP/RATSCL
            ELSE
               STEP = MAX( MININC, STEP   - 2.0D0*INCR )
            END IF
 
         END IF
 
         IF ( GOTONE  .AND.  (      .NOT. OK
     .                         .OR. STEP .GE. ENUF ) ) THEN
C
C           There are two possible ways to get to this point.
C
C           1) The current C-matrix and angular velocity are too far
C           from the last buffered matrix and angular velocity for
C           the simple interpolation method we are using to work.
C           However, the stuff in CMLAST and AVLAST were ok, so we
C           put these into the buffer and get set to start with
C           a new ETFRST and STEP.
C
C           2) Step is greater than or equal to a "reasonable" fraction
C           of the total interval that needs to be covered.
C
C           In either case, we need to store the last saved C-matrix
C           and angular velocity vector.
C
 
            QCOUNT = QCOUNT + 1
            TQ     = TQ     + 1
 
            CALL M2Q  ( CMLAST,          Q    (1,QCOUNT) )
            CALL VEQU ( AVLAST,          AVEL (1,QCOUNT) )
            CALL SCE2T( SCID,   ETLAST,  TICKS(  QCOUNT) )
C
C           Move the last ending C-matrix and angular velocity
C           into the slots used for the beginning items in
C           interpolation.
C
            CALL VEQUG ( CMLAST, 9,      CMATRX )
            CALL VEQU  ( AVLAST,         AV     )
C
C           Call the report facility to note the last time for
C           which we have an interpolation set.
C
            CALL REPORT ( ETLAST, TQ )
 
C
C           We adjust the increment value, so that it never gets
C           too far out of hand.
C
            IF ( VNORM(AV) .GT. 0 ) THEN
               INCR = MAX( MININC, INCSCL*TOL/VNORM(AV) )
            ELSE
               INCR = MAX( MININC, ENUF/RATSCL )
            END IF
 
C
C           Finally adjust the stepsize back a small amount
C           so that we have a good chance of having the
C           interpolation method work next time.
C
            STEP   = MAX( MININC, STEP   - 2.0D0*INCR )
            ETFRST = ETLAST
            GOTONE = .FALSE.
 
 
         END IF
C
C        If the quaternion buffer has filled up, we need to
C        write it out into the C-Kernel we are building.
C
C        Note that you can only exercise the block inside the IF
C        statement if you also exercised the ELSE block in the IF-THEN
C        etc. block above.
C
         IF ( QCOUNT .EQ. QBUFSZ ) THEN
 
            BEGTIK =  TICKS(1)
            ENDTIK =  TICKS(QCOUNT)
 
            AVFLAG = .TRUE.
 
            NINTS     = 1
            STARTS(1) = TICKS(1)
 
            CALL CKW03 ( HANDLE, BEGTIK, ENDTIK, PLTFRM,  REF,  AVFLAG,
     .                   SEGID,  QCOUNT, TICKS,  Q,       AVEL, NINTS,
     .                   STARTS  )
 
 
            IF ( FAILED() ) THEN
               CALL CHKOUT ( 'FITCK3' )
               RETURN
            END IF
C
C           Move the last quaternion, angular velocity and tick into
C           the first slot of the buffer so there will be no gaps
C           between the next segment and the one just written.
C
            CALL VEQUG ( Q   (1,QCOUNT), 4, Q   (1,1) )
            CALL VEQU  ( AVEL(1,QCOUNT),    AVEL(1,1) )
 
            TICKS(1) = TICKS(QCOUNT)
            QCOUNT   = 1
 
         END IF
C
C        Note that ETSTEP is modified in the enclosing DO WHILE
C        loop only in the next statement.
C
         ETSTEP   = MIN ( ETFRST + STEP, ENDET )
 
      END DO
 
C
C     Since ETLAST is modified only by setting it equal to ETSTEP
C     in the loop above, and ETSTEP is set only by the assignment
C     ETSTEP = MIN ( ETFRST + STEP, ENDET ) it follows that once
C     you reach this point ETLAST = ENDET.  Also there is only
C     one place where ETLAST is set.  At that location we set
C     CMLAST and AVLAST but we have not buffered them.  We do that
C     now.
C
      QCOUNT = QCOUNT + 1
      TQ     = TQ     + 1
 
      CALL M2Q  ( CMLAST,          Q    (1,QCOUNT) )
      CALL VEQU ( AVLAST,          AVEL (1,QCOUNT) )
      CALL SCE2T( SCID,   ETLAST,  TICKS(  QCOUNT) )
C
C     There is a remote possibility that the last two TICKS values
C     are the same due to granularity of the SCLK we are using.
C
      IF ( TICKS(QCOUNT) .EQ. TICKS(QCOUNT-1) ) THEN
         QCOUNT = QCOUNT - 1
      END IF
C
C     As long as we have more than 1 orientation set, we need to
C     write a CK segment.
C
      IF ( QCOUNT .GT. 1 ) THEN
 
         AVFLAG    = .TRUE.
         NINTS     = 1
         STARTS(1) = TICKS(1)
 
         BEGTIK =  TICKS(1)
         ENDTIK =  TICKS(QCOUNT)
 
         CALL CKW03 ( HANDLE, BEGTIK, ENDTIK, PLTFRM,  REF,  AVFLAG,
     .                SEGID,  QCOUNT, TICKS,  Q,       AVEL, NINTS,
     .                STARTS  )
 
      END IF
 
      CALL CHKOUT ( 'FITCK3' )
      RETURN
 
      END
